#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# SevenSegmentDisplay demo vs. 0.2 (2023) (PySide6 version)
#
# Written by E. A. Tacao <mailto@tacao.com.br>, (C) 2020, 2023
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     (1) Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#
#     (2) Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#
#     (3)The name of the author may not be used to
#     endorse or promote products derived from this software without
#     specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

#-------------------------------------------------------------------------------------------------------------------------------

import sys, time

from PySide6.QtWidgets import QApplication, QWidget, QLabel, QFrame, QToolButton, QMainWindow, QVBoxLayout, QHBoxLayout, \
                              QGridLayout, QRadioButton, QButtonGroup, QLineEdit, QSpinBox, QColorDialog, QSlider, QCheckBox, \
                              QSpacerItem
from PySide6.QtGui import QPalette, QColor
from PySide6.QtCore import Qt, Signal, QTimer

from sevensegment import SevenSegmentDisp

#-------------------------------------------------------------------------------------------------------------------------------

class ColorSelect(QToolButton):

    ColourSelectSignal = Signal(object)

    def __init__(self, value=(0, 0, 0), parent=None):
        super().__init__(parent)
        self.SetValue(value)
        self.clicked.connect(self.OpenDialog)


    def OpenDialog(self):
        color = QColorDialog.getColor(initial=self.value, parent=None, title="Select color",
                                      options=QColorDialog.ShowAlphaChannel|QColorDialog.DontUseNativeDialog)
        if color.isValid():
            self.SetValue(color)
            self.ColourSelectSignal.emit(color)


    def SetValue(self, value):
        if type(value) is tuple:
            value = QColor(*value)
        self.setPalette(QPalette(value))
        self.setAutoFillBackground(True)
        self.value = value


    def GetValue(self):
        return self.value


    def GetValueTuple(self):
        return self.value.toTuple()


    def SetLabel(self, label):
        self.setText(label)


    def GetLabel(self):
        return self.text()

#-------------------------------------------------------------------------------------------------------------------------------

class DisplayDemo(QMainWindow):
    def __init__(self, parent):
        super().__init__(parent)

        self.parent = parent
        self.setWindowTitle("SevenSegmentDisplay Demo")

        self.widget = QWidget()
        self.layout = QVBoxLayout()
        self.widget.setLayout(self.layout)
        self.setCentralWidget(self.widget)

        layout1 = QHBoxLayout()

        self.disp = SevenSegmentDisp(self, number_of_digits=5)
        self.disp.SetSpacing(15)
        self.disp.SetTilt(9)
        self.disp.SetColors(background=QColor(0, 0, 0),
                            segment_on=QColor(1, 196, 196, 255),
                            segment_off=QColor(0, 33, 33, 255))
        self.disp.SetValue("3.14159265358979323846264338327950288419716939937510")
        self.disp.EnableDot("all", True)

        self.setStyleSheet('QMainWindow {background-color: %s}' % QColor(0, 0, 0).name())

        layout1.addWidget(self.disp)
        self.layout.addLayout(layout1)

    # events -------------------------------------------------------------------------------------------------------------------

    def closeEvent(self, evt):
        self.parent.close()


    def keyPressEvent(self, evt):
        if evt.key() == Qt.Key_Escape:
            self.close()

#-------------------------------------------------------------------------------------------------------------------------------

class title:
    def __init__(self, parent, label, layout):
        layout_ = QHBoxLayout()
        layout_.addWidget(QLabel(label))
        line_ = QFrame(parent)
        line_.setFrameShape(QFrame.HLine)
        line_.setFrameShadow(QFrame.Sunken)
        layout_.addWidget(line_, stretch=1)
        layout.addLayout(layout_)

#-------------------------------------------------------------------------------------------------------------------------------

class SevenSegmentDisplayDemoControl(QMainWindow):
    def __init__(self):
        super().__init__()

        self.setAttribute(Qt.WA_AlwaysShowToolTips, True)

        self.demo = DisplayDemo(self)

        self.setWindowTitle("SevenSegmentDisplay Demo Control")

        self.widget = QWidget()
        self.layout = QVBoxLayout()
        self.widget.setLayout(self.layout)
        self.setCentralWidget(self.widget)

        layouta = QGridLayout()

        layout1 = QVBoxLayout()
        title(self, "Demo options", layout1)

        group1 = QButtonGroup(self)

        layout1a = QHBoxLayout()
        radio1 = QRadioButton("Clock", self)
        radio1.setToolTip("Transforms the display in a digital clock")
        group1.addButton(radio1)
        layout1a.addSpacing(20)
        layout1a.addWidget(radio1)
        layout1a.addSpacing(20)
        layout1.addLayout(layout1a)
        radio1.clicked.connect(lambda checked: self.OnRadio("Clock"))

        layout1b = QHBoxLayout()
        radio2 = QRadioButton("Custom value:", self)
        radio2.setToolTip("Type a custom value to be shown")
        group1.addButton(radio2)
        radio2.setChecked(True)
        layout1b.addSpacing(20)
        layout1b.addWidget(radio2)
        self.custom_value = QLineEdit(self)
        self.custom_value.setToolTip("Type a custom value to be shown")
        self.custom_value.setText("3.14159265358979323846264338327950288419716939937510")
        self.custom_value.setCursorPosition(0)
        layout1b.addWidget(self.custom_value)
        layout1b.addSpacing(20)
        layout1.addLayout(layout1b)
        radio2.clicked.connect(lambda checked: self.OnRadio("Custom value"))
        self.custom_value.textChanged.connect(lambda value: self.OnText("Custom value", value))

        layout1c = QHBoxLayout()
        layout1c.addSpacing(40)
        layout1c.addWidget(QLabel("Number of digits:"))

        self.digits = QSpinBox()
        self.digits.setMinimum(1)
        self.digits.setMaximum(50)
        self.digits.setValue(5)
        self.digits.setToolTip("Select the number of digits")
        layout1c.addWidget(self.digits)
        layout1c.addSpacing(20)
        self.digits.valueChanged.connect(lambda value: self.OnSpin("Digits", value))

        layout1.addLayout(layout1c)

        title(self, "Colors", layout1)

        layout1d = QGridLayout()

        color_sels = (("Background:", QColor(0, 0, 0), "Changes the background color"),
                      ("Segment on:", QColor(1, 196, 196, 255), "Changes the colour of the lit segments"),
                      ("Segment off:", QColor(0, 33, 33, 255), "Changes the colour of the dimmed segments"))

        for row, (label, color, tip) in enumerate(color_sels):
            cs = ColorSelect(color, self)
            cs.setToolTip(tip)
            cs.ColourSelectSignal.connect(lambda value, label=label[:-1]: self.OnSelectColor(label, value))
            layout1d.setColumnMinimumWidth(0, 20)
            layout1d.addWidget(QLabel(label), row, 1)
            layout1d.addWidget(cs, row, 2, alignment=Qt.AlignLeft)
        layout1.addLayout(layout1d)

        line = QFrame(self)
        line.setFrameShape(QFrame.VLine)
        line.setFrameShadow(QFrame.Sunken)
        layouta.addWidget(line, 0, 1, 3, 1)

        title(self, "Geometry", layout1)

        layout2a = QGridLayout()

        spins = (("Width", 0, 1, 38, "Width of the a, d, g segments"),
                 ("Height", 0, 4, 38, "Height of the b, c, e, f segments"),
                 ("Thickness", 1, 1, 10, "Changes the segments thickness"),
                 ("Separation", 1, 4, 3, "Adjusts the gap between segments"),
                 ("Spacing", 2, 1, 15, "Changes the distance between digits"))

        for label, row, col, value, tip in spins:
            spin = QSpinBox()
            spin.setMinimum(0)
            spin.setMaximum(128)
            spin.setValue(value)
            spin.setToolTip(tip)
            setattr(self, label.lower(), spin)
            layout2a.setColumnMinimumWidth(0, 20)
            layout2a.addWidget(QLabel(label+":"), row, col)
            w = getattr(self, label.lower())
            layout2a.addWidget(w, row, col+1)
            w.valueChanged.connect(lambda value, label=label: self.OnSpin(label, value))
        for row in range(0, 2):
            layout2a.addItem(QSpacerItem(10, 10), row, 6)

        layout2a.addItem(QSpacerItem(20, 20), 4, 0)

        self.tilt = QSlider(Qt.Horizontal)
        self.tilt.setMinimum(-42)
        self.tilt.setMaximum(+42)
        self.tilt.setTickInterval(5)
        self.tilt.setTickPosition(QSlider.TicksAbove)
        self.tilt.setValue(9)
        self.tilt.setToolTip("Tilt display by n degrees")
        self.tilt_value = QLabel("9°")
        self.tilt_value.setToolTip("Tilt display by n degrees")
        layout2a.addWidget(QLabel("Tilt:"), 5, 1)
        layout2a.addWidget(self.tilt, 5, 2, 1, 3)
        layout2a.addWidget(self.tilt_value, 5, 5, alignment=Qt.AlignRight)
        self.tilt.valueChanged.connect(self.OnSlider)

        layout1.addLayout(layout2a)

        layouta.addLayout(layout1, 0, 0)

        layout4 = QVBoxLayout()

        title(self, "Dot", layout4)

        layout2c = QHBoxLayout()
        layout2c.addSpacing(20)
        self.checkbox_dot1 = QCheckBox("Enable", self)
        self.checkbox_dot1.setChecked(True)
        self.checkbox_dot1.setToolTip("Enable dots on all digits, or specified digits.\nUnused dots will appear dimmed.")
        layout2c.addWidget(self.checkbox_dot1)
        layout4.addLayout(layout2c)
        self.checkbox_dot1.clicked.connect(lambda checked: self.OnCheck("Enable dot", checked))

        group2 = QButtonGroup(self)
        layout2c0 = QHBoxLayout()
        radio3 = self.radio3 = QRadioButton("All", self)
        self.radio3.setToolTip("Enable dots on all digits.\nUnused dots will appear dimmed.")
        group2.addButton(radio3)
        radio3.setChecked(True)
        layout2c0.addSpacing(40)
        layout2c0.addWidget(radio3)
        layout4.addLayout(layout2c0)
        radio3.clicked.connect(lambda checked: self.OnRadio("All dots enable"))

        layout2c1 = QHBoxLayout()
        radio4 = self.radio4 = QRadioButton("Digits:", self)
        self.radio4.setToolTip("Enable dots on specified digits.\n"
                               "Unused dots on specified digits will appear dimmed;\n"
                               "dots on unspecified digits, used or not, won't appear at all.")
        group2.addButton(radio4)
        layout2c1.addSpacing(40)
        layout2c1.addWidget(radio4)
        self.dot_digits = QLineEdit(self)
        self.dot_digits.setToolTip("Specify a list of digits that should show a dot;\n"
                                   "items must be separated by ';', ',' or spaces.")
        layout2c1.addWidget(self.dot_digits)
        layout2c1.addSpacing(20)
        layout4.addLayout(layout2c1)
        radio4.clicked.connect(lambda checked: self.OnRadio("Dot digits enable"))
        self.dot_digits.textEdited.connect(lambda text: self._doDotColonLogic())

        layout2d = QHBoxLayout()
        layout2d.addSpacing(20)
        self.checkbox_dot2 = QCheckBox("Reserve space", self)
        self.checkbox_dot2.setToolTip("Allocate space for dots on all digits, or specified digits.\n"
                                      "Dots won't show, unless explicitly enabled.")
        layout2d.addWidget(self.checkbox_dot2)
        layout4.addLayout(layout2d)
        self.checkbox_dot2.clicked.connect(lambda checked: self.OnCheck("Reserve space dot", checked))

        group3 = QButtonGroup(self)
        layout2d0 = QHBoxLayout()
        radio5 = self.radio5 = QRadioButton("All", self)
        self.radio5.setToolTip("Allocate space for dots on all digits.\n"
                               "Dots won't show, unless explicitly enabled.")
        group3.addButton(radio5)
        radio5.setChecked(True)
        layout2d0.addSpacing(40)
        layout2d0.addWidget(radio5)
        layout4.addLayout(layout2d0)
        radio5.clicked.connect(lambda checked: self.OnRadio("All dots reserve"))

        layout2d1 = QHBoxLayout()
        radio6 = self.radio6 = QRadioButton("Digits:", self)
        self.radio6.setToolTip("Allocate space for dots on specified digits.\n"
                               "Dots won't show, unless explicitly enabled.")
        group3.addButton(radio6)
        layout2d1.addSpacing(40)
        layout2d1.addWidget(radio6)
        self.dot_digits_reserve = QLineEdit(self)
        self.dot_digits_reserve.setToolTip("Specify a list of digits that should allocate space for the dot;\n"
                                           "items must be separated by ';', ',' or spaces.")
        layout2d1.addWidget(self.dot_digits_reserve)
        layout4.addLayout(layout2d1)
        layout2d1.addSpacing(20)
        radio6.clicked.connect(lambda checked: self.OnRadio("Dot digits reserve"))
        self.dot_digits_reserve.textEdited.connect(lambda text: self._doDotColonLogic())

        title(self, "Colon", layout4)

        layout2c2 = QHBoxLayout()
        layout2c2.addSpacing(20)
        self.checkbox_colon1 = QCheckBox("Enable", self)
        self.checkbox_colon1.setToolTip("Enable colons on all digits, or specified digits.\nUnused colons will appear dimmed.")
        layout2c2.addWidget(self.checkbox_colon1)
        layout4.addLayout(layout2c2)
        self.checkbox_colon1.clicked.connect(lambda checked: self.OnCheck("Enable colon", checked))

        group22 = QButtonGroup(self)
        layout2c02 = QHBoxLayout()
        radio32 = self.radio32 = QRadioButton("All", self)
        self.radio32.setToolTip("Enable colons on all digits.\nUnused colons will appear dimmed.")
        group22.addButton(radio32)
        radio32.setChecked(True)
        layout2c02.addSpacing(40)
        layout2c02.addWidget(radio32)
        layout4.addLayout(layout2c02)
        radio32.clicked.connect(lambda checked: self.OnRadio("All colons enable"))

        layout2c12 = QHBoxLayout()
        radio42 = self.radio42 = QRadioButton("Digits:", self)
        self.radio42.setToolTip("Enable colons on specified digits.\n"
                                "Unused colons on specified digits will appear dimmed;\n"
                                "colons on unspecified digits, used or not, won't appear at all.")
        group22.addButton(radio42)
        layout2c12.addSpacing(40)
        layout2c12.addWidget(radio42)
        self.colon_digits = QLineEdit(self)
        self.colon_digits.setToolTip("Specify a list of digits that should show a colon;\n"
                                     "items must be separated by ';', ',' or spaces.")
        layout2c12.addWidget(self.colon_digits)
        layout2c12.addSpacing(20)
        layout4.addLayout(layout2c12)
        radio42.clicked.connect(lambda checked: self.OnRadio("Colon digits enable"))
        self.colon_digits.textEdited.connect(lambda text: self._doDotColonLogic())

        layout2d2 = QHBoxLayout()
        layout2d2.addSpacing(20)
        self.checkbox_colon2 = QCheckBox("Reserve space", self)
        self.checkbox_colon2.setToolTip("Allocate space for colons on all digits, or specified digits.\n"
                                        "Colons won't show, unless explicitly enabled.")
        layout2d2.addWidget(self.checkbox_colon2)
        layout4.addLayout(layout2d2)
        self.checkbox_colon2.clicked.connect(lambda checked: self.OnCheck("Reserve space colon", checked))

        group32 = QButtonGroup(self)
        layout2d02 = QHBoxLayout()
        radio52 = self.radio52 = QRadioButton("All", self)
        self.radio52.setToolTip("Allocate space for colons on all digits.\n"
                                "Colons won't show, unless explicitly enabled.")
        group32.addButton(radio52)
        radio52.setChecked(True)
        layout2d02.addSpacing(40)
        layout2d02.addWidget(radio52)
        layout4.addLayout(layout2d02)
        radio52.clicked.connect(lambda checked: self.OnRadio("All colons reserve"))

        layout2d12 = QHBoxLayout()
        radio62 = self.radio62 = QRadioButton("Digits:", self)
        self.radio62.setToolTip("Allocate space for colons on specified digits.\n"
                                "Colons won't show, unless explicitly enabled.")
        group32.addButton(radio62)
        layout2d12.addSpacing(40)
        layout2d12.addWidget(radio62)
        self.colon_digits_reserve = QLineEdit(self)
        self.colon_digits_reserve.setToolTip("Specify a list of digits that should allocate space for the colon;\n"
                                             "items must be separated by ';', ',' or spaces.")
        layout2d12.addWidget(self.colon_digits_reserve)
        layout2d12.addSpacing(20)
        layout4.addLayout(layout2d12)
        radio62.clicked.connect(lambda checked: self.OnRadio("Colon digits reserve"))
        self.colon_digits_reserve.textEdited.connect(lambda text: self._doDotColonLogic())

        layouta.addLayout(layout4, 0, 2)

        layouta.setColumnMinimumWidth(0, 300)
        layouta.setColumnMinimumWidth(2, 300)

        self.layout.addLayout(layouta)

        self.dot_digits.setEnabled(False)
        self.dot_digits_reserve.setEnabled(False)

        self.radio32.setEnabled(False)
        self.radio42.setEnabled(False)

        self.colon_digits.setEnabled(False)

        self.radio52.setEnabled(False)
        self.radio62.setEnabled(False)
        self.colon_digits_reserve.setEnabled(False)

        self.clock_timer = QTimer(self)
        self.clock_timer.timeout.connect(self.OnClockTimer)
        self.tc = 1


    def _doDotColonLogic(self):
        if self.checkbox_dot1.isChecked():
            if self.radio3.isChecked():
                digits = "all"
            else:
                digits = []
                for i in self.dot_digits.text().replace(",", " ").replace(";", " ").split():
                    try:
                        digits.append(int(i))
                    except ValueError:
                        pass

            self.demo.disp.EnableDot("all", False)
            self.demo.disp.EnableDot(digits)

        else:
            self.demo.disp.EnableDot("all", False)

        if self.checkbox_dot2.isChecked():
            if self.radio5.isChecked():
                digits = "all"
            else:
                digits = []
                for i in self.dot_digits_reserve.text().replace(",", " ").replace(";", " ").split():
                    try:
                        digits.append(int(i))
                    except ValueError:
                        pass

            self.demo.disp.ReserveDot("all", False)
            self.demo.disp.ReserveDot(digits)

        else:
            self.demo.disp.ReserveDot("all", False)

        if self.checkbox_colon1.isChecked():
            if self.radio32.isChecked():
                digits = "all"
            else:
                digits = []
                for i in self.colon_digits.text().replace(",", " ").replace(";", " ").split():
                    try:
                        digits.append(int(i))
                    except ValueError:
                        pass

            self.demo.disp.EnableColon("all", False)
            self.demo.disp.EnableColon(digits)

        else:
            self.demo.disp.EnableColon("all", False)

        if self.checkbox_colon2.isChecked():
            if self.radio52.isChecked():
                digits = "all"
            else:
                digits = []
                for i in self.colon_digits_reserve.text().replace(",", " ").replace(";", " ").split():
                    try:
                        digits.append(int(i))
                    except ValueError:
                        pass

            self.demo.disp.ReserveColon("all", False)
            self.demo.disp.ReserveColon(digits)

        else:
            self.demo.disp.ReserveColon("all", False)


    # events -------------------------------------------------------------------------------------------------------------------

    def closeEvent(self, evt):
        pass


    def keyPressEvent(self, evt):
        if evt.key() == Qt.Key_Escape:
            self.close()

    # signals ------------------------------------------------------------------------------------------------------------------

    def OnRadio(self, msg):
        if msg == "Clock":
            self.custom_value.setEnabled(False)
            self.digits.setValue(6)
            self.demo.disp.SetNumberOfDigits(6)
            self.digits.setEnabled(False)
            self.checkbox_dot1.setChecked(False)
            self.checkbox_dot2.setChecked(False)
            self.radio32.setEnabled(True)
            self.radio42.setEnabled(True)
            self.colon_digits.setEnabled(True)
            self.checkbox_colon1.setChecked(True)
            self.radio42.setChecked(True)
            self.colon_digits.setText("1 3")
            self.checkbox_colon2.setChecked(False)
            self._doDotColonLogic()
            self.clock_timer.start(500)

        elif msg == "Custom value":
            self.custom_value.setEnabled(True)
            self.digits.setEnabled(True)
            self.clock_timer.stop()
            self.demo.disp.SetValue(self.custom_value.text())

        elif msg == "All dots enable":
            self.dot_digits.setEnabled(False)

        elif msg == "Dot digits enable":
            self.dot_digits.setEnabled(True)

        elif msg == "All dots reserve":
            self.dot_digits_reserve.setEnabled(False)

        elif msg == "Dot digits reserve":
            self.dot_digits_reserve.setEnabled(True)

        elif msg == "All colons enable":
            self.colon_digits.setEnabled(False)

        elif msg == "Colon digits enable":
            self.colon_digits.setEnabled(True)

        elif msg == "All colons reserve":
            self.colon_digits_reserve.setEnabled(False)

        elif msg == "Colon digits reserve":
            self.colon_digits_reserve.setEnabled(True)

        self._doDotColonLogic()


    def OnCheck(self, msg, status):
        if msg == "Enable dot":
            self.radio3.setEnabled(status)
            self.radio4.setEnabled(status)
            self.dot_digits.setEnabled(status)

        elif msg == "Reserve space dot":
            self.radio5.setEnabled(status)
            self.radio6.setEnabled(status)
            self.dot_digits_reserve.setEnabled(status)

        elif msg == "Enable colon":
            self.radio32.setEnabled(status)
            self.radio42.setEnabled(status)
            self.colon_digits.setEnabled(status)

        elif msg == "Reserve space colon":
            self.radio52.setEnabled(status)
            self.radio62.setEnabled(status)
            self.colon_digits_reserve.setEnabled(status)

        self._doDotColonLogic()


    def OnSlider(self, value):
        self.tilt_value.setText(str(value)+"°")
        self.demo.disp.SetTilt(value)


    def OnSpin(self, msg, value):
        if msg == "Digits":
            self.demo.disp.SetNumberOfDigits(value)
            self._doDotColonLogic()

        elif msg == "Spacing":
            self.demo.disp.SetSpacing(value)

        elif msg == "Width":
            self.demo.disp.SetGeometry(width=value)

        elif msg == "Height":
            self.demo.disp.SetGeometry(height=value)

        elif msg == "Thickness":
            self.demo.disp.SetGeometry(thickness=value)

        elif msg == "Separation":
            self.demo.disp.SetGeometry(separation=value)


    def OnText(self, msg, value):
        if msg == "Custom value":
            self.demo.disp.SetValue(value)


    def OnSelectColor(self, msg, value):
        if msg == "Background":
            self.demo.disp.SetColors(background=value)
            self.demo.setStyleSheet('QMainWindow {background-color: %s}' % value.name())
        elif msg == "Segment on":
            self.demo.disp.SetColors(segment_on=value)
        elif msg == "Segment off":
            self.demo.disp.SetColors(segment_off=value)


    def OnClockTimer(self):
        self.tc = -self.tc
        t = time.localtime(time.time())
        value = time.strftime("%H:%M:%S", t)
        value = value.replace(":", "") if self.tc > 0 else value

        if value != self.demo.disp.value:
            self.value = value
            self.demo.disp.SetValue(value)

#-------------------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':
    app = QApplication(sys.argv)
    control = SevenSegmentDisplayDemoControl()
    control.show()
    control.demo.show()
    app.exec()


#
##
### eof
