#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# SevenSegmentDisplay vs. 0.2 (2023) (PySide6 version)
#
# Written by E. A. Tacao <mailto@tacao.com.br>, (C) 2020, 2023
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     (1) Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#
#     (2) Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#
#     (3)The name of the author may not be used to
#     endorse or promote products derived from this software without
#     specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

#-------------------------------------------------------------------------------------------------------------------------------

import os, sys, math
tan = lambda x: math.tan(math.radians(x))
cos = lambda x: math.cos(math.radians(x))

from PySide6.QtCore import QPoint, Qt, QRect, QSize, QLineF, QTimer, QRectF, QPointF
from PySide6.QtGui import QColor, QPainter, QPen, QPixmap, QBrush, QPainterPath, QTransform
from PySide6.QtWidgets import QApplication, QLabel, QMainWindow, QWidget, QHBoxLayout, QSizePolicy, QVBoxLayout

#-------------------------------------------------------------------------------------------------------------------------------

class BaseWindow(QLabel):
    def __init__(self, parent, shaped=QSize(10, 10)):
        QLabel.__init__(self, parent)

        if shaped is None:
            minimum_size = QSize(10, 10)
        else:
            minimum_size = shaped
        self.shaped = shaped
        self.minimum_size = minimum_size

        self.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding)
        self.setMinimumSize(minimum_size)

        self.canvas = QPixmap(minimum_size)
        self.setPixmap(QPixmap(minimum_size))
        self.buffer_constants = QPixmap(minimum_size)
        self.buffer_variables = QPixmap(minimum_size)

        self.SetBackgroundColor(QColor(255, 255, 255))
        self.SetAntiAliasingConstants(False)
        self.SetAntiAliasingVariables(False)
        self.pause_events = False


    def showEvent(self, evt):
        sz = self.size()
        self.resize(sz + QSize(1, 1))
        self.resize(sz - QSize(1, 1))
        QLabel.show(self)


    def resizeEvent(self, evt):
        if self.pause_events:
            return

        self.setAlignment(Qt.AlignHCenter|Qt.AlignVCenter)
        if self.shaped is None:
            AspectRatio = Qt.IgnoreAspectRatio
        else:
            AspectRatio = Qt.KeepAspectRatio
        canvas = self.canvas.scaled(self.width(), self.height(), AspectRatio)
        self.setPixmap(canvas)
        sz = self.pixmap().size()

        self.buffer_constants = self.buffer_constants.scaled(sz.width(), sz.height())
        self.buffer_variables = self.buffer_variables.scaled(sz.width(), sz.height())
        self.update()
        self._drawConstants()


    def _drawConstants(self):
        size = self.pixmap().size()
        self.buffer_constants.fill(Qt.transparent)
        painter = QPainter(self.buffer_constants)
        if self.aac:
            painter.setRenderHint(QPainter.Antialiasing)

        # start drawing constants...
        self.DrawConstants(painter, size)
        # ...end drawing constants

        painter.end()
        self.UpdateDrawing()


    def _drawVariables(self):
        size = self.pixmap().size()

        self.buffer_variables.fill(Qt.transparent)
        painter = QPainter(self.buffer_variables)
        if self.aav:
            painter.setRenderHint(QPainter.Antialiasing)

        # start drawing variables...
        self.DrawVariables(painter, size)
        # ...end drawing variables

        painter.end()

    # public methods -----------------------------------------------------------------------------------------------------------

    def UpdateDrawing(self):
        if self.pause_events:
            return

        size = self.pixmap().size()
        canvas = QPixmap(size)
        canvas.fill(self.bgclr)
        painter = QPainter(canvas)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.drawPixmap(0, 0, self.buffer_constants)
        self._drawVariables()
        painter.drawPixmap(0, 0, self.buffer_variables)

        self.setPixmap(canvas)
        painter.end()


    def PauseEvents(self, pause=True):
        self.pause_events = pause


    def ResumeEvents(self):
        self.PauseEvents(False)


    def SendSizeEvent(self):
        sz = self.size()
        self.resize(sz + QSize(1, 1))
        self.resize(sz - QSize(1, 1))


    def DrawConstants(self, painter, size):
        # to be overriden...
        pass


    def DrawVariables(self, painter, size):
        # to be overriden...
        pass


    def SetBackgroundColor(self, bgclr):
        self.bgclr = bgclr
        self.setStyleSheet('QMainWindow {background-color: %s}' % self.bgclr.name())
        self.SendSizeEvent()


    def GetBackgroundColor(self):
        return self.bgclr


    def SetAntiAliasingConstants(self, flag=True):
        self.aac = flag


    def SetAntiAliasingVariables(self, flag=True):
        self.aav = flag

#-------------------------------------------------------------------------------------------------------------------------------

class QPainterPathEx(QPainterPath):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


    def DrawArcCenter(self, x, y, radius, startAngle, arcLenght):
        rect = QRectF(0, 0, radius*2, radius*2)
        rect.moveCenter(QPointF(x, y))
        self.arcTo(rect, startAngle, arcLenght)

#-------------------------------------------------------------------------------------------------------------------------------

class Digit:
    def __init__(self, parent):
        self.parent         = parent
        self.thickness      = 10
        self.rwidth         = 38
        self.rheight        = 38
        self.sep            = 3

        self.reserve_dot    = False
        self.enable_dot     = False
        self.reserve_colon  = False
        self.enable_colon   = False

        self.dot_coord      = (0, 0)
        self.colon_coord    = (0, 0, 0, 0)

        self.drawing_params = type("drawing_params", (), {})

        self._opts = {
                      "0" : "1111110",
                      "1" : "0110000",
                      "2" : "1101101",
                      "3" : "1111001",
                      "4" : "0110011",
                      "5" : "1011011",
                      "6" : "1011111",
                      "7" : "1110000",
                      "8" : "1111111",
                      "9" : "1111011",
                      "A" : "1110111",
                      "B" : "0011111",
                      "C" : "1001110",
                      "D" : "0111101",
                      "E" : "1001111",
                      "F" : "1000111",
                      "H" : "0110111",
                      "L" : "0001110",
                      "O" : "0011101",
                      "P" : "1100111",
                      "R" : "0000101",
                      "-" : "0000001",
                      " " : "0000000",
                      "°" : "1100011"
                     }

        self.value = " "


    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        # Values will be converted to uppercase strings.
        # Invalid values will be silently converted to empty spaces.

        # Examples of valid values:
        #   3 or "3" - will display a '3'

        #   "A"     - all decimal and hex digits ar allowed, but also some other
        #             characters as per the _opts dict.

        #   "3."    - will display a '3' with a trailing dot (if EnableDot
        #             is True, which it is, by default)

        #   "3:"    - will display a '3' with a trailing colon (if EnableColon
        #             is True, which it is, by default)

        #   "3.:" or "3:."
        #           - will display a '3' with trailing dot and a trailing colon
        #             (if EnableDot and EnableColon are True -- they are, by
        #             default)

        #   "." or " ."
        #           - will display an empty display with a trailing dot

        #   ":" or " :"
        #           - will display an empty display with a trailing colon

        #   ".:" or ":." or " .:" or " :."
        #           - will display an empty display with a trailing dot
        #             and a trailing colon

        if type(value) is not str:
            self._value = str(value)
        else:
            self._value = value

        self.has_dot   = "." in self._value
        self.has_colon = ":" in self._value

        self._value = self._value.replace(".", "").replace(":", "")

        if len(self._value):
            self._value = self._value[0]
        else:
            self._value = " "

        self.segments = self._opts.get(self._value, self._opts[" "])


    def Recalc(self, x_offset=0):
        path = QPainterPathEx()

        spaths = []
        for s in ["a", "b", "c", "d", "e", "f", "g", "dot", "colon"]:
            fn = getattr(self, "GetPath_"+s)
            spath = fn(x_offset)

            if s == "dot" and (self.reserve_dot or self.enable_dot):
                path.addPath(spath)

            elif s == "colon" and (self.reserve_colon or self.enable_colon):
                path.addPath(spath)

            elif s not in ["dot", "colon"]:
                path.addPath(spath)

            spaths.append(spath)

        transform = QTransform()

        # tilt
        transform = transform.shear(-tan(self.parent.tilt), 0)
        transform = transform.scale(2-cos(self.parent.tilt), 1)

        mpath = transform.map(path)
        bx, by, bw, bh = mpath.boundingRect().getRect()

        self.drawing_params.path      = path
        self.drawing_params.spaths    = spaths
        self.drawing_params.transform = transform
        self.drawing_params.bw        = bw
        self.drawing_params.bh        = bh


    def GetPath_a(self, x_offset=0):
        ox = self.thickness+self.sep+x_offset
        oy = self.thickness
        path = QPainterPathEx()
        path.moveTo(ox, oy)
        path.DrawArcCenter(ox, oy, self.thickness, 135, -45)
        pos = path.currentPosition()
        x, y = pos.x(), pos.y()
        path.lineTo(x+self.rwidth, y)
        path.DrawArcCenter(self.rwidth+ox, oy, self.thickness, 90, -45)
        path.lineTo(self.rwidth+ox, oy)
        path.closeSubpath()
        return path


    def GetPath_b(self, x_offset=0):
        ox = self.thickness+self.sep+x_offset
        oy = self.thickness
        path = QPainterPathEx()
        path.moveTo(self.rwidth+self.sep+ox, oy+self.sep)
        path.DrawArcCenter(self.rwidth+self.sep+ox, oy+self.sep, self.thickness, 45, -45)
        pos = path.currentPosition()
        x = pos.x(); y = pos.y()
        path.lineTo(x, y+self.rheight)
        path.lineTo(x-self.thickness/2, y+self.rheight+self.thickness/2)
        path.lineTo(x-self.thickness, y+self.rheight)
        path.lineTo(x-self.thickness, y)
        path.closeSubpath()
        return path


    def GetPath_c(self, x_offset=0):
        ox = self.thickness+self.sep+x_offset
        oy = self.thickness
        path = QPainterPathEx()
        x, y = self.rwidth+self.sep+ox, oy+self.rheight+self.thickness+self.sep*3
        path.moveTo(x, y)
        path.lineTo(x+self.thickness/2, y-self.thickness/2)
        path.lineTo(x+self.thickness, y)
        path.lineTo(x+self.thickness, y+self.rheight)
        path.DrawArcCenter(x, y+self.rheight, self.thickness, 0, -45)
        path.lineTo(x, y+self.rheight)
        path.closeSubpath()
        return path


    def GetPath_d(self, x_offset=0):
        ox = self.thickness+self.sep+x_offset
        oy = self.thickness
        path = QPainterPathEx()
        path.moveTo(ox, self.rheight*2+self.thickness+self.sep*4+oy)
        path.DrawArcCenter(ox, self.rheight*2+self.thickness+self.sep*4+oy, self.thickness, 225, 45)
        pos = path.currentPosition()
        x = pos.x(); y = pos.y()
        path.lineTo(x+self.rwidth, y)
        path.DrawArcCenter(x+self.rwidth, y-self.thickness, self.thickness, 270, 45)
        path.lineTo(x+self.rwidth, y-self.thickness)
        path.lineTo(x, y-self.thickness)
        path.closeSubpath()
        return path


    def GetPath_e(self, x_offset=0):
        ox = self.thickness+self.sep+x_offset
        oy = self.thickness
        path = QPainterPathEx()
        x, y = (ox-self.thickness-self.sep, oy+self.rheight+self.thickness+self.sep*3)
        path.moveTo(x, y)
        path.lineTo(x+self.thickness/2, y-self.thickness/2)
        path.lineTo(x+self.thickness, y)
        path.lineTo(x+self.thickness, y+self.rheight)
        path.DrawArcCenter(x+self.thickness, y+self.rheight, self.thickness, 225, -45)
        path.lineTo(x, y+self.rheight)
        path.closeSubpath()
        return path


    def GetPath_f(self, x_offset=0):
        ox = self.thickness+self.sep+x_offset
        oy = self.thickness
        path = QPainterPathEx()
        path.moveTo(ox-self.sep, oy+self.sep)
        path.DrawArcCenter(ox-self.sep, oy+self.sep, self.thickness, 135, 45)
        path.lineTo(ox-self.thickness-self.sep, oy+self.rheight+self.sep)
        pos = path.currentPosition()
        x = pos.x(); y = pos.y()
        path.lineTo(x+self.thickness/2, y+self.thickness/2)
        path.lineTo(x+self.thickness, y)
        path.closeSubpath()
        return path


    def GetPath_g(self, x_offset=0):
        ox = self.thickness+self.sep+x_offset
        oy = self.thickness
        path = QPainterPathEx()
        path.moveTo(ox, oy+self.rheight+self.sep*2)
        path.lineTo(ox+self.rwidth, oy+self.rheight+self.sep*2)
        path.lineTo(ox+self.rwidth+self.thickness/2, oy+self.rheight+self.sep*2+self.thickness/2)
        path.lineTo(ox+self.rwidth, oy+self.rheight+self.sep*2+self.thickness)
        path.lineTo(ox, oy+self.rheight+self.sep*2+self.thickness)
        path.lineTo(ox-self.thickness/2, oy+self.rheight+self.sep*2+self.thickness/2)
        path.closeSubpath()
        return path


    def GetPath_dot(self, x_offset=0):
        path = QPainterPathEx()
        a, b = self.thickness*2.5+self.sep*2+self.rwidth+x_offset, self.rheight*2+self.thickness+self.sep*4+self.thickness
        path.addRect(a, b, self.thickness, self.thickness)
        self.dot_coord = (a, b)
        path.closeSubpath()
        return path


    def GetPath_colon(self, x_offset=0):
        path = QPainterPathEx()
        a1 = a2 = self.thickness*3+self.sep*2+self.rwidth+x_offset
        b1 = self.thickness+self.sep+self.rheight/2-self.thickness/2
        b2 = b1+self.sep*2+self.thickness+self.rheight
        path.addRect(a1, b1, self.thickness, self.thickness)
        path.addRect(a2, b2, self.thickness, self.thickness)
        self.colon_coord  = (a1, b1, a2, b2)
        path.closeSubpath()
        return path

#-------------------------------------------------------------------------------------------------------------------------------

class SevenSegmentDisp(BaseWindow):
    def __init__(self, parent, number_of_digits=1):
        super().__init__(parent)

        self.parent = parent
        self.digits = [Digit(self) for i in range(0, number_of_digits)]
        self.drawing_params = None

        self.spacing        = 0
        self.tilt           = 0 # degrees

        self.colors = type("colors", (), {})
        self.colors.background    = QColor(0, 0, 0, 255)
        self.colors.pen_seg_on    = QColor(1, 196, 196, 255)
        self.colors.brush_seg_on  = QColor(0, 196, 196, 255)
        self.colors.pen_seg_off   = QColor(0, 33, 33, 255)
        self.colors.brush_seg_off = QColor(0, 33, 33, 255)

        self.SetAntiAliasingConstants(True)
        self.SetAntiAliasingVariables(True)

        self.SetValue("")


    def LedsOn(self, painter, on=True):
        if on:
            pen = QPen(self.colors.pen_seg_on, 0)
            painter.setPen(pen)
            painter.setBrush(self.colors.pen_seg_on)
        else:
            pen = QPen(self.colors.pen_seg_off, 0)
            painter.setPen(pen)
            painter.setBrush(self.colors.brush_seg_off)


    def LedsOff(self, painter):
        self.LedsOn(painter, False)


    def DrawConstants(self, painter, size):
        x_offsets = [0]
        for digit in self.digits:
            digit.Recalc()
            x_offsets.append(digit.drawing_params.bw)

        path = QPainterPath()
        x_offset = 0
        for i, digit in enumerate(self.digits):
            x_offset += x_offsets[i] + self.spacing
            digit.Recalc(x_offset)
            path.addPath(digit.drawing_params.path)

        transform = QTransform()

        # tilt
        transform = transform.shear(-tan(self.tilt), 0)
        transform = transform.scale(2-cos(self.tilt), 1)

        # resize
        sw, sh = size.width(), size.height()

        mpath = transform.map(path)
        bx, by, bw, bh = mpath.boundingRect().getRect()

        if self.minimum_size != QSize(bw/10, bh/10):
            self.PauseEvents()
            self.minimum_size = QSize(bw/10, bh/10)
            self.setMinimumSize(self.minimum_size)
            self.canvas = QPixmap(self.minimum_size)
            self.DrawConstants(painter, size)
            self.ResumeEvents()
            return

        p = min(sw/bw, sh/bh)
        transform = transform.scale(p, p)

        # center
        mpath = transform.map(path)
        bx, by, bw, bh = mpath.boundingRect().getRect()

        k = ((sw-bw)/2)-bx; l = ((sh-bh)/2)-by

        mtransform = transform.inverted()[0]
        center = mtransform.map(QPointF(k, l))
        transform = transform.translate(center.x(), center.y())

        self.drawing_params = [transform, x_offsets]


    def DrawVariables(self, painter, size):
        if self.drawing_params is None: return

        x_offsets = self.drawing_params[1]
        x_offset = 0
        for i, digit in enumerate(self.digits):
            x_offset += x_offsets[i]

            path_on  = QPainterPath()
            path_off = QPainterPath()

            transform = self.drawing_params[0]
            painter.setTransform(transform)

            for i in range(0, 7):
                if digit.segments[i] == "1":
                    path_on.addPath(digit.drawing_params.spaths[i])
                else:
                    path_off.addPath(digit.drawing_params.spaths[i])

            self.LedsOn(painter)
            painter.drawPath(path_on)

            self.LedsOff(painter)
            painter.drawPath(path_off)

            xd, yd   = transform.map(digit.dot_coord[0],   digit.dot_coord[1])
            xc1, yc1 = transform.map(digit.colon_coord[0], digit.colon_coord[1])
            xc2, yc2 = transform.map(digit.colon_coord[2], digit.colon_coord[3])

            m11 = transform.m11()
            m12 = transform.m12()
            m13 = transform.m13()
            m21 = transform.m21()
            m22 = transform.m22()
            m23 = transform.m23()
            m31 = transform.m31()
            m32 = transform.m32()
            m33 = transform.m33()

            transform.reset()
            painter.setTransform(transform)

            d = digit.thickness*m11*cos(self.tilt)

            if digit.enable_dot:
                self.LedsOn(painter, digit.has_dot)
                painter.drawEllipse(xd, yd, d, d)

            if digit.enable_colon:
                self.LedsOn(painter, digit.has_colon)
                painter.drawEllipse(xc1, yc1, d, d)
                painter.drawEllipse(xc2, yc2, d, d)

            transform = QTransform(m11, m12, m13, m21, m22, m23, m31, m32, m33)
            self.drawing_params[0] = transform


    def _d2d(self, digits):
        if type(digits) in (list, tuple):
            digits_ = [digit for digit in digits if digit in range(0, len(self.digits))]
        elif digits == "all":
            digits_ = range(0, len(self.digits))
        else:
            digits_ = [int(digits)]
        return digits_

    # public methods -----------------------------------------------------------------------------------------------------------

    def SetValue(self, value):
        # Parse a string to feed individual values to each of the displays
        self.value = value
        cvs = []
        i = 0
        while i < len(value):
            c0 = value[i]
            if i+1 < len(value):
                c1 = value[i+1]
            else:
                c1 = ""
            if i+2 < len(value):
                c2 = value[i+2]
            else: c2 = ""

            if c0 in [".", ":"] and c1 in [".", ":"] and c0 != c1:
                p = " " + c0 + c1
                i += 1

            elif c0 in [".", ":"]:
                p = " " + c0

            else:
                if c1 in [".", ":"] and c2 in [".", ":"] and c1 != c2:
                    p = c0 + c1 + c2
                    i += 2
                elif c1 in [".", ":"] and c2 in [".", ":"] and c1 == c2:
                    p = c0 + c1
                    i += 1
                elif c1 in [".", ":"]:
                    p = c0 + c1
                    i += 1
                else:
                    p = c0

            cvs.append(p)
            i += 1

        # Align right or truncate the value shown
        num = len(self.digits)
        if len(cvs) > num:
            cvs = cvs[:num]
        else:
            cvs = [" "] * (num - len(cvs)) + cvs

        for i in range(0, len(self.digits)):
            self.digits[i].value = cvs[i]

        self.UpdateDrawing()


    def SetNumberOfDigits(self, number_of_digits):
        if number_of_digits == len(self.digits) or number_of_digits == 0:
            return

        else:
            self.pause_events = True

            if number_of_digits > len(self.digits):
                add = number_of_digits - len(self.digits)
                geometry = self.GetGeometry()
                for i in range(0, add):
                    new_digit = Digit(self)
                    new_digit.Recalc()
                    self.digits.append(new_digit)
                self.SetGeometry(**geometry)

            else:
                sub = len(self.digits) - number_of_digits
                self.digits = self.digits[:-sub]

            self.SetValue(self.value)
            self.pause_events = False
            self.SendSizeEvent()


    def SetGeometry(self, **kwargs):
        # keywords: thickness, width, height, radius, separation
        # values: int or float
        # e.g: SetGeometry(thickness=28, separation=1)

        digits = self._d2d("all")

        for digit in digits:

            if "thickness" in kwargs.keys():
                self.digits[digit].thickness = kwargs["thickness"]

            if "width" in kwargs.keys():
                self.digits[digit].rwidth = kwargs["width"]

            if "height" in kwargs.keys():
                self.digits[digit].rheight = kwargs["height"]

            if "separation" in kwargs.keys():
                self.digits[digit].sep = kwargs["separation"]

        self.SendSizeEvent()


    def GetGeometry(self):
        digit = 0
        return {"thickness":  self.digits[digit].thickness,
                "width":      self.digits[digit].rwidth,
                "height":     self.digits[digit].rheight,
                "separation": self.digits[digit].sep}


    def SetTilt(self, value):
        self.tilt = value
        self.SendSizeEvent()


    def GetTilt(self):
        return self.tilt


    def SetColors(self, **kwargs):
        # keywords: 'background', 'segment_on', 'segment_off'
        # values: QColor
        # e.g: SetColors(segment_on=QColor(255, 0, 0), background=QColor(Qt.black)

        if "background" in kwargs.keys():
            self.colors.background = kwargs["background"]
            self.SetBackgroundColor(self.colors.background)

        if "segment_on" in kwargs.keys():
            self.colors.pen_seg_on = kwargs["segment_on"]
            self.colors.brush_seg_on = kwargs["segment_on"]

        if "segment_off" in kwargs.keys():
            self.colors.pen_seg_off = kwargs["segment_off"]
            self.colors.brush_seg_off = kwargs["segment_off"]

        self.UpdateDrawing()


    def GetColors(self):
        return {"background":  self.colors.background,
                "segment_on":  self.colors.pen_seg_on,
                "segment_off": self.colors.pen_seg_off}


    def SetSpacing(self, value):
        self.spacing = value
        self.SendSizeEvent()


    def GetSpacing(self):
        return self.spacing


    def EnableDot(self, digits, val=True):
        digits = self._d2d(digits)

        for digit in digits:
            self.digits[digit].enable_dot = val

        self.SendSizeEvent()


    def IsDotEnabled(self, digit):
        return self.digits[digit].enable_dot


    def ReserveDot(self, digits, val=True):
        digits = self._d2d(digits)

        for digit in digits:
            self.digits[digit].reserve_dot = val

        self.SendSizeEvent()


    def IsDotReserved(self, digit):
        return self.digits[digit].reserve_dot


    def EnableColon(self, digits, val=True):
        digits = self._d2d(digits)

        for digit in digits:
            self.digits[digit].enable_colon = val

        self.SendSizeEvent()


    def IsColonEnabled(self, digit):
        return self.digits[digit].enable_colon


    def ReserveColon(self, digits, val=True):
        digits = self._d2d(digits)

        for digit in digits:
            self.digits[digit].reserve_colon = val

        self.SendSizeEvent()


    def IsColonReserved(self, digit):
        return self.digits[digit].reserve_colon


#-------------------------------------------------------------------------------------------------------------------------------
# test stuff -------------------------------------------------------------------------------------------------------------------

class DigitalClock(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("DigitalClock")

        widget = QWidget()
        layout = QVBoxLayout()
        widget.setLayout(layout)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        self.setCentralWidget(widget)

        layout1 = QHBoxLayout()

        disp = self.disp = SevenSegmentDisp(self, number_of_digits=6)
        disp.EnableColon([1, 3], True)
        disp.SetSpacing(5)
        disp.SetTilt(9.8)

        disp.SetBackgroundColor(QColor(0, 0, 0))
        self.setStyleSheet('QMainWindow {background-color: %s}' % QColor(0, 0, 0).name())

        layout1.addWidget(disp)
        layout.addLayout(layout1)

        self.value = ""

        self.tc = 1
        self.timer = QTimer(self)
        self.timer.timeout.connect(self.OnTimer)
        self.timer.start(500)


    def OnTimer(self):
        self.tc = -self.tc
        t = time.localtime(time.time())
        value = time.strftime("%H:%M:%S", t)
        value = value.replace(":", "") if self.tc > 0 else value

        if value != self.value:
            self.value = value
            self.disp.SetValue(value)

#-------------------------------------------------------------------------------------------------------------------------------

if __name__ == "__main__":
    app = QApplication(sys.argv)
    import time
    DigitalClock().show()
    app.exec()


#
##
### eof
